#include "datatypes.h"
#include "unes_mapper.h"
#include "unes.h"

extern int32 num_1k_VROM_banks;
extern int32 num_8k_ROM_banks;

static uint8 regs[4];

/////////////////////////////////////////////////////////////////////
// Mapper 46
static void map46_Reset()
{
	regs[0] = regs[1] = regs[2] = regs[3] = 0;

	set_rom_banks();
	set_mirroring(NES_PPU::MIRROR_VERT);
}

static void map46_MemoryWriteSaveRAM(uint32 addr, uint8 data)
{
	regs[0] = data & 0x0F;
	regs[1] = (data & 0xF0) >> 4;
	set_rom_banks();
}

static void map46_MemoryWrite(uint32 addr, uint8 data)
{
	regs[2] = data & 0x01;
	regs[3] = (data & 0x70) >> 4;
	set_rom_banks();
}

static void map46_set_rom_banks()
{
	set_CPU_bank4(regs[0]*8+regs[2]*4+0);
	set_CPU_bank5(regs[0]*8+regs[2]*4+1);
	set_CPU_bank6(regs[0]*8+regs[2]*4+2);
	set_CPU_bank7(regs[0]*8+regs[2]*4+3);
	set_PPU_bank0(regs[1]*64+regs[3]*8+0);
	set_PPU_bank1(regs[1]*64+regs[3]*8+1);
	set_PPU_bank2(regs[1]*64+regs[3]*8+2);
	set_PPU_bank3(regs[1]*64+regs[3]*8+3);
	set_PPU_bank4(regs[1]*64+regs[3]*8+4);
	set_PPU_bank5(regs[1]*64+regs[3]*8+5);
	set_PPU_bank6(regs[1]*64+regs[3]*8+6);
	set_PPU_bank7(regs[1]*64+regs[3]*8+7);
}
/////////////////////////////////////////////////////////////////////

static void map46_getstate(SnssMapperBlock *state)
{
}

static void map46_setstate(SnssMapperBlock *state)
{
}



mapintf_t map46_intf =
{
   46, /* mapper number */
   "map46", /* mapper name */
   map46_Reset, /* init routine */
   NULL, /* vblank callback */
   NULL, /* hblank callback */
   map46_getstate, /* get state (snss) */
   map46_setstate, /* set state (snss) */
   NULL,  /*Read low*/
   NULL,  /*Read saveram or rom*/
   NULL,  /*Write high regs 0x4000-0x4017*/
   NULL,  /*Write low   0x4018-0x5FFF*/
   map46_MemoryWriteSaveRAM,  /*Write saveram*/
   map46_MemoryWrite,  /*Write rom*/
   NULL,   /*Latch FDFE*/
   NULL,   /*Latch renderscreen*/
   NULL,   /*Latch Address*/
   NULL    /*SetBarcodeValue*/   
};
