#include "datatypes.h"
#include "unes_mapper.h"
#include "unes.h"

extern int32 num_1k_VROM_banks;
extern int32 num_8k_ROM_banks;

static uint8 bank, mode;
/////////////////////////////////////////////////////////////////////
// Mapper 51
static void map51_Reset()
{
	bank = 0;
	mode = 1;
	Sync_Prg_Banks();
	parent_NES->ppu->vram_write_protect = 0;
}

static void map51_MemoryWriteSaveRAM(uint32 addr, uint8 data)
{
	mode = ((data & 0x10) >> 3) | ((data & 0x02) >> 1);
	Sync_Prg_Banks();
}

static void map51_MemoryWrite(uint32 addr, uint8 data)
{
	bank = (data & 0x0f) << 2;
	if(0xC000 <= addr && addr <= 0xDFFF)
	{
		mode = (mode & 0x01) | ((data & 0x10) >> 3);
	}
	Sync_Prg_Banks();
}

static void map51_Sync_Prg_Banks()
{
	switch(mode)
	{
	case 0:
		{
			set_mirroring(NES_PPU::MIRROR_VERT);
			set_CPU_bank3(bank | 0x2c | 3);
			set_CPU_bank4(bank | 0x00 | 0);
			set_CPU_bank5(bank | 0x00 | 1);
			set_CPU_bank6(bank | 0x0c | 2);
			set_CPU_bank7(bank | 0x0c | 3);
		}
		break;

	case 1:
		{
			set_mirroring(NES_PPU::MIRROR_VERT);
			set_CPU_bank3(bank | 0x20 | 3);
			set_CPU_bank4(bank | 0x00 | 0);
			set_CPU_bank5(bank | 0x00 | 1);
			set_CPU_bank6(bank | 0x00 | 2);
			set_CPU_bank7(bank | 0x00 | 3);
		}
		break;

	case 2:
		{
			set_mirroring(NES_PPU::MIRROR_VERT);
			set_CPU_bank3(bank | 0x2e | 3);
			set_CPU_bank4(bank | 0x02 | 0);
			set_CPU_bank5(bank | 0x02 | 1);
			set_CPU_bank6(bank | 0x0e | 2);
			set_CPU_bank7(bank | 0x0e | 3);
		}
		break;

	case 3:
		{
			set_mirroring(NES_PPU::MIRROR_HORIZ);
			set_CPU_bank3(bank | 0x20 | 3);
			set_CPU_bank4(bank | 0x00 | 0);
			set_CPU_bank5(bank | 0x00 | 1);
			set_CPU_bank6(bank | 0x00 | 2);
			set_CPU_bank7(bank | 0x00 | 3);
		}
		break;
	}
}
/////////////////////////////////////////////////////////////////////

mapintf_t map51_intf =
{
   51, /* mapper number */
   "map51", /* mapper name */
   map51_Reset, /* init routine */
   NULL, /* vblank callback */
   NULL, /* hblank callback */
   map51_getstate, /* get state (snss) */
   map51_setstate, /* set state (snss) */
   NULL,  /*Read low*/
   NULL,  /*Read saveram or rom*/
   NULL,  /*Write high regs 0x4000-0x4017*/
   NULL,  /*Write low   0x4018-0x5FFF*/
   map51_MemoryWriteSaveRAM,  /*Write saveram*/
   map51_MemoryWrite,  /*Write rom*/
   NULL,   /*Latch FDFE*/
   NULL,   /*Latch renderscreen*/
   NULL,   /*Latch Address*/
   NULL    /*SetBarcodeValue*/   
};
